﻿#ifndef SRC_HEROAPI_STD_ALLOCATOR_H_
#define SRC_HEROAPI_STD_ALLOCATOR_H_

#include <deque>
#include <queue>
#include <map>
#include <memory>
#include <set>
#include <string>
#include <sstream>
#include <unordered_map>
#include <utility>
#include <vector>

namespace hero {

#ifdef HEROAPI_UE4

template <typename T>
class hero_allocator : public std::allocator<T>
{
 public:
  typedef size_t size_type;
  typedef T* pointer;
  typedef const T* const_pointer;
  template<typename _Tp1>

  struct rebind
  {
    typedef hero_allocator<_Tp1> other;
  };

  pointer allocate(size_type n, const void *hint = 0)
  {
    return reinterpret_cast<pointer>(FMemory::Malloc(n * sizeof(T)));
  }

  void deallocate(pointer p, size_type /* size_type */)
  {
    return FMemory::Free(p);
  }

  hero_allocator() throw() : std::allocator<T>() { }
  hero_allocator(const hero_allocator &a) throw() : std::allocator<T>(a) { }

  template <class U>
  hero_allocator(const hero_allocator<U> &a) throw() : std::allocator<T>(a) {}
  ~hero_allocator() throw() {}
};

typedef std::basic_stringstream<char, std::char_traits<char>, hero_allocator<char>> stringstream;
typedef std::basic_string<char, std::char_traits<char>, hero_allocator<char>> string;
typedef std::basic_ostringstream<char, std::char_traits<char>, hero_allocator<char>> ostringstream;

template<typename _Ty>
using vector = std::vector<_Ty, hero_allocator<_Ty>>;

template<typename _Ty>
using deque = std::deque<_Ty, hero_allocator<_Ty>>;

template<typename _Ty>
using queue = std::queue<_Ty, deque<_Ty>>;

template<typename _Ty>
using set = std::set<_Ty, std::less<_Ty>, hero_allocator<_Ty>>;

template<typename _Kty, typename _Ty>
using map = std::map<_Kty, _Ty, std::less<_Kty>, hero_allocator<std::pair<const _Kty, _Ty>>>;

template<typename _Kty, typename _Ty>
using unordered_map =
  std::unordered_map<_Kty, _Ty, std::hash<_Kty>, std::equal_to<_Kty>, hero_allocator<std::pair<const _Kty, _Ty>>>;

#endif // HEROAPI_UE4

} // hero namespace

#endif // SRC_HERO_STD_ALLOCATOR_H_